#pragma once

#include <stddef.h>

typedef struct _Array Array;

typedef void   (*FreeFunc)    (void *element);
typedef double (*ValueFunc)   (const void *element);
typedef int    (*CompareFunc) (const void *a, const void *b);

// Fundamental functions
Array * array_new         (FreeFunc    func);
Array * array_ref         (Array      *array);
void    array_unref       (Array      *array);

void    array_add         (Array      *array,
                           const void *val);
void    array_remove_fast (Array      *array,
                           size_t      index);
void    array_remove      (Array      *array,
                           size_t      index);
void    array_set         (Array      *array,
                           size_t      index,
                           const void *val);

// Getters    
const void * array_get          (const Array *array,
                                 size_t       index);
size_t       array_get_length   (const Array *array);
size_t       array_get_capacity (const Array *array);

// Misc
void array_sort (Array      *array,
                 CompareFunc func);
