#define _GNU_SOURCE // For asprintf

#include <stdio.h>
#include <stdlib.h>

#include "array.h"

static void
print_array (Array *array)
{
    size_t length = array_get_length (array);
    printf ("Array at %p is of length %zu\n", array, length);
    printf ("Array has capacity %zu\n", array_get_capacity (array));

    for (size_t i = 0; i < length; i++)
    {
        const char *str = (const char *) array_get (array, i);
        printf ("Array member %zu has value %s\n", i, str);
    }
}

static Array *
create_test_array (size_t start,
                   size_t end)
{
    Array *array = array_new (free);

    for (size_t i = start; i < end; i++)
    {
        char *str;
        if (asprintf (&str, "String #%zu", i) < 0)
            exit (EXIT_FAILURE);
        array_add (array, str);
    }

    return array;
}

static void
remove_array_items_fast (Array  *array,
                         size_t  start,
                         size_t  num)
{
    for (int i = 0; i < num; i++)
        array_remove_fast (array, start);
}


static void
remove_array_items (Array  *array,
                    size_t  start,
                    size_t  num)
{
    for (size_t i = 0; i < num; i++)
        array_remove (array, start);
}

int
main (void)
{
    printf ("Slow\n----\n");

    Array *array = create_test_array (0, 100);
    print_array (array);

    remove_array_items (array, 20, 70);
    print_array (array);

    array_unref (array);

    printf ("Fast\n----\n");

    array = create_test_array (0, 100);
    print_array (array);

    remove_array_items_fast (array, 20, 70);
    print_array (array);

    return EXIT_SUCCESS;
}
